@@ -77,6 +77,10 @@ class Address(dict):
             return f"{prefix}{index}"
         return self.address
 
+    @property
+    def is_labeled(self):
+        return bool(self["label"])
+
     def __str__(self):
         return self.address
 
@@ -173,6 +177,13 @@ class AddressList(dict):
         if need_save:
             self.save()
 
+    def max_index(self, change=False):
+        return max(
+            -1,
+            -1,
+            *[addr.index for addr in self.values() if addr.change == change],
+        )
+
     def max_used_index(self, change=False):
         return max(
             -1,
